from .tsgenerator import TimeSeriesGenerator
import pandas as pd  # type: ignore
from typing import Union
import numpy as np


def timeseries2df(
    data: Union[np.array], p: int, n: int, s: int = -1
) -> pd.DataFrame:  # , category_seasonal, test_split = .33):
    """Interface for `TimeSeriesGenerator`, transforming the generator into a `pd.DataFrame`

    Returns:
        p int: autoregressive parameter
        n int: lenght of `y`
        s int: seasonal parameter
    """

    # Define ts
    ts = TimeSeriesGenerator(data, p, n, s)

    # Iterate over
    features, target = [], []
    for i, (X, y) in enumerate(ts):
        features.append(X)
        target.append(y)

    # Set columns
    s_cols = [f"y(ts{s})_{i}" for i in range(1, n + 1)] if s > 0 else []
    x_cols = s_cols + [f"y(t-{i})" for i in range(p)][::-1]
    y_cols = [f"y(t+{i})" for i in range(1, n + 1)]

    # Create DataFrame
    dftime = pd.concat(
        [pd.DataFrame(features, columns=x_cols), pd.DataFrame(target, columns=y_cols)],
        axis=1,
    )

    return dftime



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tabular_time_series']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.3.2,<2.0.0', 'pre-commit>=2.14.1,<3.0.0']

setup_kwargs = {
    'name': 'tabular-time-series',
    'version': '0.2.32',
    'description': '',
    'long_description': '# Tabular Time Series\n\n## Summary\n\nThis repo was created as I did not find a function able to transform a time-series (1D) into a tabular format (X, y).\n\n## Usage\n\n### TimeSeriesGenerator\n\nThe docstring is as follows. Given a 1D array `data = [0, 1, 2, 3, 4, 5, 6]`, generates `X, y` following the parameters `p`(autoregressive), `s` (seasonal) and `n` (lenght of y).\n\nTherefore, it makes it possible to train a neural network (e.g.) that 2 autoregressive entries (e.g. `p = 2`) and predicts the next two (`n = 2`) using 2 (`n = 2`) entries with lag 4 (`s = 4`).\n\n```python\n>> data = [0, 1, 2, 3, 4, 5, 6]\n>> p, n = 2, 2\n>> ts = TimeSeriesGenerator(data, p, n)\n>> for X, y in ts:\n...    print(X, y)\n    [0, 1] [2, 3]\n    [1, 2] [3, 4]\n    [2, 3] [4, 5]\n    [3, 4] [5, 6]\n>> p, n, s = 2, 2, 4\n>> ts = TimeSeriesGenerator(data, p, n, s)\n>> for X, y in ts:\n...    # both y have their respective seasonal entry\n...    print(data.index(y[0]) - data.index(X[0]) == s, data.index(y[1]) - data.index(X[1]) == s)\n...    print(X, y)\n    [0, 1, 2, 3] [4, 5]\n    [1, 2, 3, 4] [5, 6]\n```\n\n### timeseries2df\n\nConsidering that many times a batch array is needed for training, `timeseries2df` can be used to generate a `pandas` DataFrame that will contain columns in the format:\n\n```python\n>>> from tabular_time_series.tsdf import timeseries2df\n>>> data = list(range(10))\n>>> p, n, s = 2, 2, 4\n>>> df = timeseries2df(data, p, n, s)\n>>> df\n   y(ts4)_1  y(ts4)_2  y(t-1)  y(t-0)  y(t+1)  y(t+2)\n0         0         1       2       3       4       5\n1         1         2       3       4       5       6\n2         2         3       4       5       6       7\n3         3         4       5       6       7       8\n4         4         5       6       7       8       9\n```\n',
    'author': 'Felipe Whitaker',
    'author_email': 'felipewhitaker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/felipewhitaker/tabular-time-series',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



from collections.abc import Iterator
from typing import Union, List, Tuple


class TimeSeriesGenerator(Iterator):
    def __init__(
        self, data: Union[list], p: int, n: int, s: int = -1
    ) -> Tuple[List, List]:
        """
        Iterator that receives `time series data` (rows, cols) and can
        be iterated over, returning `X` like ([s + n] + p,) and `y` like
        (n,) such that:

        >> np.where(y == y[0])[0].item() - np.where(X == X[0])[0].item()  == s
        >> np.where(y == y[0])[0].item() - np.where(X == X[-1])[0].item() == 1

        Thus

        >> data = [0, 1, 2, 3, 4, 5, 6]
        >> p, n = 2, 2
        >> ts = TimeSeriesGenerator(data, p, n)
        >> for X, y in ts:
        ...    print(X.shape, y.shape)
        ...    print(X, y)
            [0, 1] [2, 3]
            [1, 2] [3, 4]
            [2, 3] [4, 5]
            [3, 4] [5, 6]
        >> p, n, s = 2, 2, 4
        >> ts = TimeSeriesGenerator(data, p, n, s)
        >> for X, y in ts:
        ...    diff = np.where(data == y[0])[0].item() - np.where(data == X[0])[0].item()
        ...    print(X.shape, y.shape, diff) == (n + p,) (n,) s
        ...    print(X, y)
            [0, 1, 2, 3] [4, 5]
            [1, 2, 3, 4] [5, 6]

        Parameters:
        ----------
        data : np.array of shape (n,)
        p : int
        n : int
        s : int

        Returns:
        -------
        None
        """

        super().__init__()

        assert n >= 1, "`n` was set to zero: shouldn't some `y` be predicted?"
        assert (
            p >= 0 or s >= 0
        ), "Neither `p` nor `s` were set: which data will be used to predict `y`?"
        assert (
            len(data) - n - p - (n if s > 0 else 0) >= 0
        ), "Impossible to generate even one instance"
        assert (
            len(data) - s > 0
        ), f"Given data of len {len(data)} can not generate seasonal of {s}"

        self.S = -1
        if s > 0:
            assert s >= p + n, "`s` and `p` values shouldn't be overlapping"
            self.S = s

        self.data = list(data)

        self.p = p
        self.n = n

        self.s = 0

        self.curr = s - p if s > 0 else 0
        return

    def __len__(self):
        return len(self.data) - self.n - self.p

    def __getitem__(self, i):
        if self.curr > len(self):
            raise IndexError

        ys = []
        if self.S > -1:
            ys = self.data[self.s : self.s + self.n]
            self.s += 1

        ar = self.data[i : i + self.p]  # autoregressive
        yt = self.data[i + self.p : i + self.p + self.n]  # y for prediction
        return ys + ar, yt

    def __next__(self):
        try:
            X, y = self[self.curr]
        except IndexError:
            raise StopIteration()
        self.curr += 1
        return X, y
